<?php
/**
 * Core Design Ajax Vote plugin for Joomla! 1.5
 * @author		Daniel Rataj, <info@greatjoomla.com>
 * @package		Joomla
 * @subpackage	Content
 * @category	Plugin
 * @version		1.0.6
 * @copyright	Copyright (C) 2007 - 2010 Great Joomla!, http://www.greatjoomla.com
 * @license		http://www.gnu.org/copyleft/gpl.html GNU/GPL 3
 * 
 * This file is part of Great Joomla! extension.   
 * This extension is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This extension is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

// Import library dependencies
jimport('joomla.plugin.plugin');

/**
 * Core Design Ajax Vote plugin
 *
 * @author		Daniel Rataj <info@greatjoomla.com>
 * @package		Core Design
 * @subpackage	Content
 */
class plgContentCdAjaxVote extends JPlugin
{
	/**
	 * Constructor
	 *
	 * For php4 compatability we must not use the __constructor as a constructor for plugins
	 * because func_get_args ( void ) returns a copy of all passed arguments NOT references.
	 * This causes problems with cross-referencing necessary for the observer design pattern.
	 *
	 * @access	protected
	 * @param	object		$subject The object to observe
	 * @since	1.0
	 */
	function plgContentCdAjaxVote(&$subject)
	{
		parent::__construct($subject);

		// load plugin parameters
		$this->plugin = &JPluginHelper::getPlugin('content', 'cdajaxvote');
		$this->params = new JParameter($this->plugin->params);

		// load language
		JPlugin::loadLanguage('plg_content_cdajaxvote', JPATH_ADMINISTRATOR);
	}

	/*
	 * Joomla! onAfterDispatch() function
	 */
	function onAfterDispatch() {

		$task = JRequest::getCmd('cdajaxvote', '', 'post');

		switch ($task) {
			case 'vote':
				$this->vote();
				break;
			default:
				return;
				break;
		}

	}

	/**
	 * Joomla! onBeforeDisplayContent() function
	 */
	function onAfterDisplayContent(&$article, &$params, $limitstart=0)
	{
		$document = &JFactory::getDocument(); // set document for next usage

		if ($document->getType() != 'html') return; // prevent script loading in PDF or RSS output
		
		// disable section, category or article if required
		jimport('joomla.utilities.arrayhelper');

		$disablesection = explode(',', trim($this->params->get('disablesection' , '')));
		JArrayHelper::toInteger($disablesection);

		$disablecategory = explode(',', trim($this->params->get('disablecategory' , '')));
		JArrayHelper::toInteger($disablecategory);

		$disablearticle = explode(',', trim($this->params->get('disablearticle' , '')));
		JArrayHelper::toInteger($disablearticle);

		if ($article->sectionid && in_array($article->sectionid, $disablesection)) return;
		if ($article->catid && in_array($article->catid, $disablecategory)) return;
		if ($article->id && in_array($article->id, $disablearticle)) return;
		
		$tmpl = '';
		
		if (isset($article->rating_count) && $params->get( 'show_vote' ) && !$params->get( 'popup' )) {
			
			// Scriptegrator check
			if (!defined('_JSCRIPTEGRATOR')) {
				JError::raiseNotice('', JText::_('CDAJAXVOTE_ENABLE_SCRIPTEGRATOR'));
				return;
			} else {
				$message = JScriptegrator::check('1.3.7', 'jquery', 'site');
				if ($message) {
					JError::raiseNotice('', $message);
					return;
				}
			}
			
			$live_path = JURI::root(true) . '/'; // define live site
			
			// add CSS stylesheet and JavaScript file
			$document->addStyleSheet( $live_path . 'plugins/content/cdajaxvote/css/cdajaxvote.css', 'text/css' );
			$document->addScript($live_path . 'plugins/content/cdajaxvote/utils/js/ui.stars.js');
			$document->addScript($live_path . 'plugins/content/cdajaxvote/utils/js/jquery.stars.js');
			
			$starcontainer = 'vote_' . $article->id;
			
			$user = &JFactory::getUser();
			$user_type = (string )strtolower($user->usertype);
			$guest = $user->guest;
			
			$cancelShow = 'false';
			if ($article->rating) {
				// reset vote privileges
				if ($this->resetVote()) {
					$cancelShow = 'true';
				}
			}
			
			// if no article rating, set 0
			if (!$article->rating) $article->rating = 0;
			// if no article rating count, set 0
			if (!$article->rating_count) $article->rating_count = 0;
			
			$disabled = 'false';
			if ($this->params->get('disableguests', 'no') == 'yes') {
				if ($guest) $disabled = 'true';
			}
			
			$callback_script = '';
			if ($disabled == 'true') {
				$callback_script = "
					alert('" . JText::_('CDAJAXVOTE_LOGINFIRST') . "'); $('#$starcontainer').stars('select', 0);
					$('#$starcontainer').stars('select', $article->rating);
					return false;
				";
			}
			
			$starimage = $this->params->get('star_image', 'ui-stars-star');
			if ($starimage == '-1') $starimage = 'ui-stars-star';
			
			$intro_only = $params->get( 'intro_only' );
			if ($intro_only) {
				$intro_only = 'true';
			} else {
				$intro_only = 'false';
			}
			
			$init_js = "
			<script type=\"text/javascript\">
	        <!--
			jQuery(document).ready(function($){
				if ($.ui.stars && $.cdajaxvote) {
					$('#$starcontainer').stars({
						cancelShow: $cancelShow,
						showTitles: true,
						cancelTitle: '" . JText::_('CDAJAXVOTE_RESET') . "',
						starClass: '$starimage',
						disabled: $intro_only,
						callback: function(ui, type, value) {
							$callback_script
							$('#$starcontainer').stars('select', value);
							var instance = $('#$starcontainer').data('stars');
							var currvalue = instance.options.value;
							
							$.ajax({
								type: 'POST',
								timeout: 5000,
								data: 'cdajaxvote=vote&rate=' + value + '&aid=' + $article->id,
								beforeSend: function() {
									$.cdajaxvote.loading($('#$starcontainer'));
								},
								success: function(msg) {
									switch(msg) {
										case 'reset':
											$.cdajaxvote.vote($('#$starcontainer'), 'reset');
											break;
										case '':
											$.cdajaxvote.vote($('#$starcontainer'), '+');
											break;
										default:
											alert(msg);
											$('#$starcontainer').stars('select', $article->rating);
										break;
									}
								},
								complete: function() {
									$.cdajaxvote.loading($('#$starcontainer'));
								}
							});
						}
					});
					$('#$starcontainer').stars('select', $article->rating);
				}
			});
			// -->
			</script>";
			$document->addCustomTag($init_js);
			
			$tmpl .= $this->setTmpl($starcontainer, $article->rating_count);
		}
		
		return $tmpl;

	}
	
	/**
	 * Set displayed template
	 * 
	 * @param $starcontainer
	 * @param $rating_count
	 * 
	 * @return string		Template.
	 */
	function setTmpl($starcontainer, $rating_count = 0) {
		
		$inputs = $this->params->get('voting_number', 5);
		
		$tmpl = '';
		$tmpl .= '
		<div class="cdajaxvote">
		<span id="stars-cap"></span>
		<div id="' . $starcontainer . '" class="rating">
		<form action="index.php">';
		for ($i=1; $i<=$inputs; $i++) {
			$tmpl .= '<input type="radio" name="newrate" value="' . $i . '" title="' . $i . '" />';
		}
		$tmpl .= '</form>
		</div>
		<div class="rating_count">(<span>' .intval($rating_count) . '</span> - ' . JText::_('CDAJAXVOTE_VOTE_COUNT') . ')</div>
		<div style="clear: both;"></div>
		</div>';
		return $tmpl;
	}
	
	/**
	 * Check privileges to reset the article voting.
	 * 
	 * @return boolean		True if user has access to reset the vote.
	 */
	function resetVote() {
		$user = &JFactory::getUser();
		$user_type = (string )strtolower($user->usertype);
		
		if ($user_type == 'super administrator' or $user_type == 'administrator') {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Method to determine a hash for anti-spoofing variable names
	 *
	 * @return	string	Hashed var name
	 */
	function getToken() {
		jimport('joomla.utilities.utility');
		$token = JUtility::getToken();
		return $token;
	}

	/**
	 * Global close wrapper
	 *
	 * @param $msg		message to display
	 *
	 * @return void
	 */
	function close($msg = '') {
		global $mainframe;
		$mainframe->close($msg);
	}
	
	/**
	 * Vote
	 * 
	 * @return string		Message if error.
	 */
	function vote() {
		
		$aid = JRequest::getInt('aid', 0, 'post');
		$rate = JRequest::getInt('rate', 0, 'post');
		
		// rate is 0, reset voting for article
		if ($rate == 0) {
			$this->resetVoting();
		}
		
        $db = &JFactory::getDBO();        

        if ($rate >= 1 && $rate <= 5) {
            $userIP = $_SERVER['REMOTE_ADDR'];

            $query = 'SELECT *' . ' FROM #__content_rating' . ' WHERE content_id = ' . $aid;
            $db->setQuery($query);
            $rating = $db->loadObject();

            if (!$rating) {
                // There are no ratings yet, so lets insert our rating
                $query = 'INSERT INTO #__content_rating ( content_id, lastip, rating_sum, rating_count )' .
                    ' VALUES ( ' . $aid . ', ' . $db->Quote($userIP) . ', ' . $rate .
                    ', 1 )';
                $db->setQuery($query);
                if (!$db->query()) {
                    $this->close(JText::_('CDAJAXVOTE_DATABASE_ERROR'));
                } else {
                    $this->close();
                }
            } else {
                if ($userIP != ($rating->lastip)) {
                    // We weren't the last voter so lets add our vote to the ratings totals for the article
                    $query = 'UPDATE #__content_rating' .
                        ' SET rating_count = rating_count + 1, rating_sum = rating_sum + ' . $rate .
                        ', lastip = ' . $db->Quote($userIP) . ' WHERE content_id = ' . $aid;
                    $db->setQuery($query);
                    if (!$db->query()) {
                        $this->close(JText::_('CDAJAXVOTE_DATABASE_ERROR'));
                    } else {
                        $this->close();
                    }
                } else {
                    $this->close(JText::_('CDAJAXVOTE_YOU_HAVE_ALREADY_VOTED'));
                }
            }
        } else {
            $this->close();
        }
    }
    
    /**
     * Reset voting
     * 
     * @return string		Message if error, "reset" if success.
     */
	function resetVoting()
    {
    	if (!$this->resetVote()) $this->close($this->close(JText::_('CDAJAXVOTE_ACCESSERROR')));
    	
    	$aid = JRequest::getInt('aid', 0, 'post');
        $db = &JFactory::getDBO();

        $query = 'SELECT *' . ' FROM #__content_rating' . ' WHERE content_id = ' . $aid;
        $db->setQuery($query);
        $rating = $db->loadObject();

        if (!$rating) {
            $this->close(JText::_('CDAJAXVOTE_VOTE_DELETE_EMPTY'));
        } else {
            $query = 'DELETE FROM #__content_rating' . ' WHERE content_id = ' . $aid .
                ' LIMIT 1';
            $db->setQuery($query);
            if (!$db->query()) {
                $this->close(JText::_('CDAJAXVOTE_DATABASE_ERROR'));
            } else {
                $this->close('reset');
            }
        }
    }
}
?>